# Управление ресурсными группами
[![Jira][jira_logo]][ticket]

---

* [Описание](#описание)
* [Создание](#создание-и-изменение)
* [Удаление](#удаление)
* [Добавления ролей](#добавления-ролей-в-ресурсные-группы)
* [Удаление ролей](#удаление-ролей-из-ресурсных-групп)

### Описание
Управлние ресурсными группами в Greenplum реализовано через Ansible.  

К сожалению в Ansible нет готового модуля для работы с Greenplum. Был написан [модуль Ansible][greenplum.py], для управления ресурсными группами.  
Управление группами происходит через передачу `Json` параметра с описанием групп.  

В зависимости  от требуемого действие список параметров может быть сокращен до только названия групп/группы или наоброт иметь макисмальное описание (при создании) .

Действия указываются через тэги Ansible:
* `present` -- Создаёт группы, если параметры группы отличаются от текущих, текущие параметры будут заменены на новые
* `absent` -- Удаляет ресурсную группы, при этом будут сброшены роли
* `alter` -- Добавление/Удаление ролей из ресурсных групп. При удалении в качестве ресурсной группы указывается None, при добавлении имя группы.  

##### Пример параметров при добавлении:
```json
{
    "groups_name":[{
        "rsgname": "env_1",
            "concurrency": "3",
            "memory_limit": "5",
            "cpu_rate_limit": "4"
            }]
}
```

---

### Создание и изменение
```ansible
ansible-playbook -i files_for_installer/hosts ansible-playbook ansible/resource_group.yml -e @files_for_installer/common_vars.yml -e '{"groups_name":[{"rsgname": "env_2", "concurrency": "1", "memory_limit": "1","cpu_rate_limit": "1"},{"rsgname": "test_group_ansible", "concurrency": "1", "memory_limit": "1", "cpu_rate_limit": "2"}, {"rsgname": "env_1", "concurrency": "3", "memory_limit": "5","cpu_rate_limit": "4"}]}' --tags=present
```

### Удаление
***Удаление со всеми параметрами группы***
```ansible
ansible-playbook -i files_for_installer/hosts ./resource_group.yml -e @files_for_installer/common_vars.yml -e '{"groups_name":[{"rsgname": "env_1", "concurrency": "3", "memory_limit": "5","cpu_rate_limit": "4"}]}' --tags=absent
```
***Удаление двух групп***
```ansible
ansible-playbook -i files_for_installer/hosts ./resource_group.yml -e @files_for_installer/common_vars.yml -e '{"groups_name":[{"rsgname": "env_1"}, {"rsgname": "env_2"}]}' --tags=absent
```
***Удаление одной группы***
```ansible
ansible-playbook -i files_for_installer/hosts ./resource_group.yml -e @files_for_installer/common_vars.yml -e '{"groups_name":[{"rsgname": "env_1", }]}' --tags=absent
```

### Добавления ролей в ресурсные группы
```ansible
ansible-playbook -i files_for_installer/hosts ./resource_group.yml -e @files_for_installer/common_vars.yml  -e '{"groups_name":[{"rsgname": "default_group", "role": "env_1"}]}' --tags=alter
```

### Удаление ролей из ресурсных групп  
```ansible
ansible-playbook -i files_for_installer/hosts ./resource_group.yml -e @files_for_installer/common_vars.yml  -e '{"groups_name":[{"rsgname": "None", "role": "env_1"}]}' --tags=alter
```

[greenplum.py]: library/greenplum.py
[ticket]: http://jira.sch.ocrv.com.rzd:8080/browse/BDP-4058 "Задача"
[jira_logo]: https://confluence.corp.myservices.digital/images/logo/default-space-logo.svg
