import yaml
import requests
from requests.auth import HTTPBasicAuth
import json
import sys
import time
import configparser
import pathlib
import logging


LEVEL = 'INFO'
FORMAT = "[{levelname:^10s}] - {message:s}"
logging.basicConfig(format=FORMAT, style='{', level=LEVEL)
# # Проверим есть ли конфигфайл
# try:
#     import config
# except ModuleNotFoundError as error:
#     if str(error) == "No module named 'config'":
#         print('Не хватает конфигурационного файла config.py')
#     # print(error)
#     sys.exit(1)

# # Проверим есть ли стартовый файл
# if not os.path.isfile(config.start_hosts_file):
#     print(f'Ошибка, не нашел стартовый файл {config.start_hosts_file}')
#     sys.exit(1)

def get_ini( file):
        ini = configparser.ConfigParser()
        ini.read(file)
        return ini
      
current_path = str(pathlib.Path(__file__).parent.absolute())
      
config_file = f'{current_path}/config.ini'
start_hosts_file = f'{current_path}/start_hosts.yml'
files_for_installer_path = f'{current_path}/files_for_installer'

config = get_ini(config_file)

ambari_login = 'admin'
ambari_password = 'admin'
ambari_port = 8000


logging.info(f'')

def return_hosts():
    try:
        with open(start_hosts_file) as f:
            return yaml.safe_load(f)
    except FileNotFoundError:
        logging.info(f'Ошибка, не нашел стартовый файл {start_hosts_file}')
        sys.exit(1)

def load_animation(text, sleep ):
    animation = "|/-\\"
    anicount = 0
    counttime = 0        

    while (counttime != sleep * 10):
        time.sleep(0.1) 
        sys.stdout.write("\r"+ text + ' ' + animation[anicount])
        sys.stdout.flush()
        anicount = (anicount + 1)% 4
        counttime += 1

def provisioning_status(id):
    url = f'http://{ambari_host_port}/api/v1/clusters/datapark/requests/{id}'
    r = requests.get(url, auth=basic,  headers=headers).json()["Requests"]
    return r

def waiting_provisioning(msg='Идет установка Ambari', t_wait=10, id=1):
  msg = f'{msg} '
  while True:
    provisioning_info = provisioning_status(id)
    progress_percent = provisioning_info['progress_percent']
    request_status = provisioning_info['request_status']
    task_count = provisioning_info['task_count']
    completed_task_count = provisioning_info['completed_task_count']
    failed_task_count = provisioning_info['failed_task_count']
    tasks_runing = task_count - completed_task_count

    if tasks_runing == 0 and request_status != 'PENDING':
      if failed_task_count > 0:
        logging.error(f'Установка завершена с ошибками!')
        logging.error(f'Провалилось задач: {failed_task_count}')
        sys.exit(1)
      else:
        logging.info('Установка завершена успешно!')
        return request_status

    if request_status == 'PENDING':
      load_animation(f'Ждем запуска таски установки', 5)
      
    msg = f'Процент выполнения: {round(progress_percent, 2)}% Задач в работе: {tasks_runing}'
    if request_status == "FAILED":
      msg = f'{msg}, Провалилось задач: {failed_task_count}'
    load_animation(f'{msg}'.ljust(50, ' '), t_wait)


hosts = return_hosts()


repo_url = config['nexus']['yum']

ambari_host_port = ''
ambari_host = ''
for host in hosts:
  if host['ansible_groups']:
    if 'repo_node' in host['ansible_groups']:
      ambari_host = f"{host['name']}.{config['global']['domain']}"
      if config['global']['env_prefix'] != '':
        ambari_host = f"{config['global']['env_prefix']}-{host['name']}.{config['global']['domain']}"

  ambari_host_port = f"{ambari_host}:{str(ambari_port)}"

basic = HTTPBasicAuth(ambari_login, ambari_password)
url = f'http://{ambari_host_port}/api/v1/stacks/HDP/versions/3.1/repository_versions/'
headers = {'X-Requested-By': 'ambari', 'Content-Type': 'application/x-www-form-urlencoded'}

logging.info("Запуск настройки сервисов Hadoop Ambari Platform")

# Принтануть URLs, если указан ключ 'manual' для запуска этого скрипта
# Требуется для ручной установки кластера во время настройки урлов репозитория
if 'manual' in sys.argv:
  logging.info(f"{repo_url}")
  sys.exit(0)

data = {
  "RepositoryVersions": {
    "repository_version": "3.1.4.0-315",
    "display_name": "HDP-3.1"
  },
  "operating_systems": [
    {
      "OperatingSystems": {
        "os_type": "redhat7"
      },
      "repositories": [
        {
            "href": f"http://{ambari_host_port}/api/v1/stacks/HDP/versions/3.1/repository_versions/1/operating_systems/redhat7/repositories/HDP-3.1",
            "Repositories": {
              "base_url": f"{repo_url}",
              "os_type": "redhat7",
              "repo_id": "HDP-3.1",
              "repo_name": "HDP"
          }
        },
        {
          "href": f"http://{ambari_host_port}/api/v1/stacks/HDP/versions/3.1/repository_versions/1/operating_systems/redhat7/repositories/HDP-UTILS-1.1.0.22",
          "Repositories": {
            "base_url": f"{repo_url}",
            "os_type": "redhat7",
            "repo_id": "HDP-UTILS-1.1.0.22",
            "repo_name": "HDP-UTILS"
          }
        }
      ]
    }
  ]
}

r = requests.post(url, auth=basic, headers=headers, data=json.dumps(data))
status_code_repo = r.status_code

status_code_config = False
if status_code_repo == 201 or status_code_repo == 500:
  logging.info('Конфигурирование репозитория')
  url = f'http://{ambari_host_port}/api/v1/stacks/HDP/versions/3.1/repository_versions/1/operating_systems/redhat7/repositories/HDP-3.1/'
  r = requests.get(url, auth=basic)
  status_code_config = r.status_code
else:
  logging.error('Ошибка конфигурирования репозитория')
  logging.error('status_code', status_code_repo)
  sys.exit(1)
  

status_code_bp = False
if status_code_config == 200:
  logging.info("Репозиторий сконфигурирован")
  logging.info("Загрузка Blueprint")
  url = f'http://{ambari_host_port}/api/v1/blueprints/datapark?validate_topology=false'

  data = open(f'{files_for_installer_path}/blueprint.json', 'rb')
  r = requests.post(url, auth=basic, headers=headers, data=data)
  logging.info(r.text)
  status_code_bp = r.status_code
else:
  logging.error('Ошибка конфигурирования репозитория')
  logging.error('status_code_config', status_code_config)
  sys.exit(1)


status_start = False
if status_code_bp == 201:
  logging.info("Запуск, установка и настройка сервисов Hadoop Ambari Platform")
  url = f'http://{ambari_host_port}/api/v1/clusters/datapark'

  data = open(f'{files_for_installer_path}/clustertemplate.json', 'rb')
  r = requests.post(url, auth=basic, headers=headers, data=data)
  if r.status_code == 202:
    status_start = r.json()['Requests']['status']
    id_job = r.json()['Requests']['id']
else:
  logging.error('Ошибка загрузки Blueprint')
  logging.error('status_code_bp', status_code_bp)
  sys.exit(1)


if status_start == 'Accepted':
  logging.info('Отправляю команду на запуск установки Ambari.')
  logging.info(f'Статус установки можно посмотреть тут: http://{ambari_host_port}')
  waiting_provisioning(id_job)
else:
  logging.error('Ошибка установки и настройки сервисов Hadoop Ambari Platform')
  sys.exit(1)
