#!/bin/bash
set -eu
INSTALLER_FILES_DIR="./files_for_installer"
STATUS_FILES="$INSTALLER_FILES_DIR/stat"
INSTALLER_ANSIBLE_DIR="./ansible"
ANSIBLE_VARS="@$INSTALLER_FILES_DIR/common_vars.yml"
VENV_PATH="./installer-venv"
VENV_CMD="source $VENV_PATH/bin/activate"
ANSIBLE_PBOOK="ansible-playbook"

# Сценарии
PREPARE_INSTALLER_STEP="./prepare_installer_node.sh:prepare_installer_node"
PREPARE_INSTALLER_VENV="./setup_venv.sh:venv_setup"
ACTIVATE_VENV="$VENV_CMD:always_run"
PREPARE_CONFIGS="python3 main.py:files_for_installer"
ANS_HOSTS_PREPARE="$ANSIBLE_PBOOK $INSTALLER_ANSIBLE_DIR/hosts_prepare/main.yml -e $ANSIBLE_VARS:hosts_prepare"
ANS_DOCK="$ANSIBLE_PBOOK $INSTALLER_ANSIBLE_DIR/docker/main.yml -e $ANSIBLE_VARS:docker"
ANS_PLATFORM="$ANSIBLE_PBOOK $INSTALLER_ANSIBLE_DIR/platform/main.yml -e $ANSIBLE_VARS:platform"
ANS_GP="$ANSIBLE_PBOOK $INSTALLER_ANSIBLE_DIR/gp/main.yml -e $ANSIBLE_VARS:gp"
AMBARI_SETUP="python3 install_ambari.py:ambari_setup"
ANS_AIRFLOW="$ANSIBLE_PBOOK $INSTALLER_ANSIBLE_DIR/airflow_post_install/main.yml -e $ANSIBLE_VARS:airflow_post_install"
ANS_PXF="$ANSIBLE_PBOOK $INSTALLER_ANSIBLE_DIR/pxf_config/main.yml -e $ANSIBLE_VARS:pxf_config"
ANS_HDFS_CONNECTOR="$ANSIBLE_PBOOK $INSTALLER_ANSIBLE_DIR/hdfs_connector_setup/main.yml -e $ANSIBLE_VARS:hdfs_connector_setup"
ANS_MON="$ANSIBLE_PBOOK $INSTALLER_ANSIBLE_DIR/monitoring/main.yml -e $ANSIBLE_VARS:monitoring"

mkdir -p $STATUS_FILES
declare -a installer_steps=(
							"$PREPARE_INSTALLER_STEP"
							"$PREPARE_INSTALLER_VENV"
							"$ACTIVATE_VENV"
							"$PREPARE_CONFIGS"
							"$ANS_HOSTS_PREPARE"
							"$ANS_PLATFORM"
							"$ANS_GP"
							"$AMBARI_SETUP"
							"$ANS_PXF"
							"$ANS_HDFS_CONNECTOR"
							"$ANS_MON"
                            )

echo "### Запущена установка платформы КХД" 

for action in "${installer_steps[@]}"
do
    STEP_CMD="$(echo "$action" | grep -o '.*:' | tr -d ':')"
    STEP_FILE="$(echo "$action" | grep -o ':.*' | tr -d ':')"

	if [[ ! -f "$STATUS_FILES/$STEP_FILE" ]]
		then echo "### Выполняю \"$STEP_CMD\"" 
			$STEP_CMD
		# shellcheck disable=SC2181
		if [[ $? -gt "0" ]]
			then "Возникла ошибка, перезапустите скрипт установки после её устранения"
			exit 1
			else 
				echo "$STEP_FILE" | grep -qw 'always_run' || touch "$STATUS_FILES/$STEP_FILE"
		fi
	fi

done
echo "### Установка успешно завершена" 
exit 0
