#!/bin/bash
set -e

ANS_TEST_HOST="${1}"
ANS_TEST_SERVER="${2}"

if [ -z "$ANS_TEST_SERVER" ]; then EXTRA_SERVER_VARS=" "
  else
    if grep -qw "$ANS_TEST_SERVER" files_for_installer/hosts
      then
        echo "Запуск iperf3 сервера на $ANS_TEST_SERVER"
        EXTRA_SERVER_VARS="-e server=$ANS_TEST_SERVER"
      else
        echo "Ошибка, хост \"$ANS_TEST_SERVER\" не найден в inventory"
        exit 1
    fi
fi

if [ -z "$ANS_TEST_HOST" ]; then EXTRA_CLIENT_VARS=" "
  else
    if grep -qw "$ANS_TEST_HOST" files_for_installer/hosts
      then
        echo "Запуск perf_test на $ANS_TEST_HOST"
        EXTRA_CLIENT_VARS="-e host=$ANS_TEST_HOST"
      else
        echo "Ошибка, хост \"$ANS_TEST_HOST\" не найден в inventory"
        exit 1
    fi
fi

set -u
EXTRA_VARS="$EXTRA_SERVER_VARS $EXTRA_CLIENT_VARS"
ANSIBLE_PBOOK="ansible-playbook"
INSTALLER_ANSIBLE_DIR="./ansible"
VENV_PATH="./installer-venv"
VENV_CMD="source $VENV_PATH/bin/activate"
export ANSIBLE_LOG_PATH="./perf_test.log"
echo > $ANSIBLE_LOG_PATH

# shellcheck disable=SC2086
$VENV_CMD && $ANSIBLE_PBOOK $INSTALLER_ANSIBLE_DIR/perf_test/main.yml $EXTRA_VARS

echo "Итоговая статистика для $ANS_TEST_HOST:"
CPU_MAX_TR_STAT=$(grep sysbench_cpu.stdout_lines -A 25 $ANSIBLE_LOG_PATH | grep ' events per second:' | tr '"' ' ' | awk '{print $4}')
MEM_SEQ_READ_STAT=$(grep 'Running memory speed test with the following options' -A 20 $ANSIBLE_LOG_PATH | grep -wA 15 'read' | grep 'transferred' | tr '")(' ' ' | awk '{print $4}')
MEM_SEQ_WRITE_STAT=$(grep 'Running memory speed test with the following options' -A 40 $ANSIBLE_LOG_PATH | grep -wA 15 'write' | grep 'transferred' | tr '")(' ' ' | awk '{print $4}')
HDD_SEQ_READ_STAT=$(grep -A 15 'Doing sequential read test' $ANSIBLE_LOG_PATH | tr '"' ' ' | grep 'read, MiB/s:' | awk '{print $3}')
HDD_SEQ_WRITE_STAT=$(grep -A 15 'Doing sequential write (creation) test' $ANSIBLE_LOG_PATH | tr '"' ' ' | grep 'written, MiB/s:' | awk '{print $3}')
HDD_SEQ_RND_READ_STAT=$(grep -A 25 'Doing random r/w test' $ANSIBLE_LOG_PATH | tr '"' ' ' | grep 'read, MiB/s:' | awk '{print $3}')
HDD_SEQ_RND_WRITE_STAT=$(grep -A 25 'Doing random r/w test' $ANSIBLE_LOG_PATH | tr '"' ' ' | grep 'written, MiB/s:' | awk '{print $3}')
NET_SEND_STAT=$(grep -A 20 'item=iperf.stdout_lines' $ANSIBLE_LOG_PATH | grep sender | awk '{print $7}')
NET_RCV_STAT=$(grep -A 20 'item=iperf.stdout_lines' $ANSIBLE_LOG_PATH | grep receiver | awk '{print $7}')
echo '------------------------------------------------------'
echo "CPU 1 core:           $CPU_MAX_TR_STAT events/s"
echo "RAM seq.read:         $MEM_SEQ_READ_STAT MiB/s"
echo "RAM seq.write:        $MEM_SEQ_WRITE_STAT MiB/s"
echo "HDD seq.read:         $HDD_SEQ_READ_STAT MiB/s"
echo "HDD seq.write:        $HDD_SEQ_WRITE_STAT MiB/s"
echo "HDD rnd.r/w.read:     $HDD_SEQ_RND_READ_STAT MiB/s"
echo "HDD rnd.r/w.write:    $HDD_SEQ_RND_WRITE_STAT MiB/s"
echo "Net upload:           $NET_SEND_STAT Mbits/sec"
echo "Net download:         $NET_RCV_STAT Mbits/sec"
echo '------------------------------------------------------'
echo "perf_test завершён успешно"

exit 0
